'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));

function create(fn) {
  var state = {
    listeners: [],
    current: fn(function (merge) {
      if (typeof merge === 'function') merge = merge(state.current);
      state.current = Object.assign({}, state.current, merge);
      state.listeners.forEach(function (listener) {
        return listener(state.current);
      });
    }, function () {
      return state.current;
    })
  };
  return [// useStore
  function (selector, dependencies) {
    var selected = selector ? selector(state.current) : state.current; // Using functional initial b/c selected itself could be a function

    var _React$useState = React.useState(function () {
      return selected;
    }),
        slice = _React$useState[0],
        set = _React$useState[1];

    var sliceRef = React.useRef();
    React.useEffect(function () {
      return void (sliceRef.current = slice);
    }, [slice]);
    React.useEffect(function () {
      var ping = function ping() {
        // Get fresh selected state
        var selected = selector ? selector(state.current) : state.current; // Refresh local slice, functional initial b/c selected itself could be a function

        if (!shallowEqual(sliceRef.current, selected)) set(function () {
          return selected;
        });
      };

      state.listeners.push(ping);
      return function () {
        return state.listeners = state.listeners.filter(function (i) {
          return i !== ping;
        });
      };
    }, dependencies || [selector]); // Returning the selected state slice

    return selected;
  }, {
    subscribe: function subscribe(fn) {
      state.listeners.push(fn);
      return function () {
        return state.listeners = state.listeners.filter(function (i) {
          return i !== fn;
        });
      };
    },
    getState: function getState() {
      return state.current;
    },
    destroy: function destroy() {
      return state.listeners = [], state.current = {};
    }
  }];
}
var hasOwn = Object.prototype.hasOwnProperty;

function is(x, y) {
  if (x === y) {
    return x !== 0 || y !== 0 || 1 / x === 1 / y;
  } else {
    return x !== x && y !== y;
  }
}

function shallowEqual(objA, objB) {
  if (is(objA, objB)) {
    return true;
  }

  if (typeof objA !== 'object' || objA === null || typeof objB !== 'object' || objB === null) {
    return false;
  }

  var keysA = Object.keys(objA);
  var keysB = Object.keys(objB);

  if (keysA.length !== keysB.length) {
    return false;
  }

  for (var i = 0; i < keysA.length; i++) {
    if (!hasOwn.call(objB, keysA[i]) || !is(objA[keysA[i]], objB[keysA[i]])) {
      return false;
    }
  }

  return true;
}

module.exports = create;
